package com.pspdf;

import static org.junit.Assert.assertTrue;

import com.pspdfkit.api.DocumentEditor;
import com.pspdfkit.api.PSPDFKit;
import com.pspdfkit.api.PSPDFKitInitializeException;
import com.pspdfkit.api.PdfDocument;
import com.pspdfkit.api.basic.Rotation;
import com.pspdfkit.api.providers.FileDataProvider;
import org.json.JSONArray;
import org.junit.BeforeClass;
import org.junit.Test;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

/**
 * Unit test for simple App.
 */
public class AppTest 
{
    private void savePdfFileToDisk(String fileName, DocumentEditor editor)
    {
        // Export the document to a file path.
        try {
            File file = File.createTempFile(fileName, ".pdf");
            System.out.println(file.getAbsolutePath());
            editor.saveDocument(new FileDataProvider(file));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    @BeforeClass
    public static void initialSetup() throws PSPDFKitInitializeException {
        PSPDFKit.initializeTrial();
    }

    /*
    How to merge PDF pages
    The PS PDF Kit PDF API allows us to combine multiple PDF files.
    The scenario is that you have a PDF file “personal-letter.pdf” and you want to add a cover from another PDF file “coverPage.pdf” as the first page of your PDF.
    High-level steps of this demo are given below:
    1. Open two PDF files from the assets included in the project(personal-letter.pdf, coverPage.pdf)
    2. Create a DocumentEditor based on the PDF file in which we want to merge another PDF file.
    3. Combine the PDF files using the DcumentEditor.ImportDocument method of PS PDF Kit.
    4. Save the merged PDF file to the disk, as a new PDF.`
    * */
    @Test
    public void mergePDFs()
    {
        FileDataProvider providerPersonalLetter = new FileDataProvider(
                new File("Assets/personal-letter.pdf"));
        PdfDocument personalLetter = PdfDocument.open(providerPersonalLetter);

        FileDataProvider providerCoverPage = new FileDataProvider(
                new File("Assets/coverPage.pdf"));
        PdfDocument cover = PdfDocument.open(providerCoverPage);

        DocumentEditor letterEditor = personalLetter.createDocumentEditor();
        letterEditor.importDocument(0, DocumentEditor.IndexPosition.BeforeIndex, providerCoverPage);

        try {
            File file = File.createTempFile("personal-letter-with-cover", ".pdf");
            System.out.println(file.getAbsolutePath());
            letterEditor.saveDocument(new FileDataProvider(file));
        }catch (IOException ioex)
        {
            System.out.println("Unable to save file. " + ioex.toString());
        }
    }

/*
*
    How to Rotate PDF pages
    Rotating the pages of a PDF document is again a very simple operation.
    We’re simply going to use the objects of FileDataProvider, PdfDocument, and DocumentEditor.
    High-level steps of this demo are given below:
    Open a PDF file from the assets included in the project(coverPage.pdf).
    Create an editor to modify the opened PDF in memory.
    Rotate the page of the PDF using DocumentEditor.RotatePages method.
    Save the results in a new PDF file to the disk as a new PDF file.
*
* */
    @Test
    public void rotatePDF()
    {
        FileDataProvider providerPersonalLetter = new FileDataProvider(
                new File("Assets/coverPage.pdf"));
        PdfDocument personalLetter = PdfDocument.open(providerPersonalLetter);

        DocumentEditor letterEditor = personalLetter.createDocumentEditor();

        Set<Integer> setOfPagesToRotate = new HashSet<Integer>(Arrays.asList(0));

        letterEditor.rotatePages(setOfPagesToRotate, Rotation.Degrees90);

        savePdfFileToDisk("rotatedLetter", letterEditor);
    }

    /*
    How to duplicate pages of a PDF
    The duplication of PDF pages is achieved through our trusty friend DocumentEditor and
    the process is going to be more or less the same as rotating pages. Only difference will be that
    we’ll call a different PS PDF Kit API method.

    High-level steps of this demo are given below:
    Open a PDF file from the assets included in the project(playground.pdf).
    Create an editor to modify the opened PDF in memory.
    Duplicate the page of the PDF using DocumentEditor.DuplicatePages method.
    Save the results in a new PDF file to the disk as a new PDF file.

    * */
    @Test
    public void duplicatePDFPages()
    {
        FileDataProvider providerPDFFile = new FileDataProvider(
                new File("Assets/playground.pdf"));
        PdfDocument playGround = PdfDocument.open(providerPDFFile);

        DocumentEditor pdfEditor = playGround.createDocumentEditor();

        Set<Integer> setOfPagesToDuplicate = new HashSet<Integer>(Arrays.asList(0));

        pdfEditor.duplicatePages(setOfPagesToDuplicate);

        savePdfFileToDisk("fileWithDuplicatePages", pdfEditor);
    }

    /*
    How to move or rearrange pages in a PDF
    It is possible to change the sequence of pages inside a PDF file.
    For example, move the first page after the last one or move the fifth page to the sixth one.
    There can be different scenarios for moving pages of a PDF file around. Some examples are listed below:
    Moving the first page of a PDF file to the end of the PDF file.
    Moving the last page of the PDF file to the beginning of the PDF file.
    Moving a page of a PDF file from the start to the middle of the PDF file.
    Moving a page from the end of the file to the middle of the PDF file.
    Moving a page from the middle of the PDF file to the beginning of the PDF file
    Moving a page from the middle of the PDF file to the end of the PDF file
    There are many more such examples but I believe you get the point.
    Let us write some code to see these scenarios in work.

    High-level steps of this demo are given below:
    Open a PDF file from the assets included in the project(playground.pdf).
    Create an editor to modify the opened PDF in memory.
    Move pages using documentEditor.movePages
    Save the results in a new PDF file to the disk as a new PDF file.
     */
    @Test
    public void movePDFPages()
    {
        FileDataProvider providerPDFFile = new FileDataProvider(
                new File("Assets/playground.pdf"));
        PdfDocument playGround = PdfDocument.open(providerPDFFile);
        DocumentEditor pdfEditor = playGround.createDocumentEditor();

        // move first page to the last
        Set<Integer> setOfPagesToMove = new HashSet<Integer>(Arrays.asList(0));
        pdfEditor.movePages(setOfPagesToMove, Math.toIntExact(playGround.getPageCount() - 1), DocumentEditor.IndexPosition.AfterIndex);
        savePdfFileToDisk("moveFirstPDFPageToLast", pdfEditor);

        // move first page to the last
        setOfPagesToMove = new HashSet<Integer>(
                Arrays.asList(Math.toIntExact(playGround.getPageCount() - 1)));
        pdfEditor = playGround.createDocumentEditor();
        pdfEditor.movePages(setOfPagesToMove, 0, DocumentEditor.IndexPosition.BeforeIndex);
        savePdfFileToDisk("moveLastPDFPageToFirst", pdfEditor);

        // move first page to the middle
        setOfPagesToMove = new HashSet<Integer>(
                Arrays.asList(Math.toIntExact(0)));
        pdfEditor = playGround.createDocumentEditor();
        pdfEditor.movePages(setOfPagesToMove,
                (Math.toIntExact(playGround.getPageCount()) / 2) -1
                , DocumentEditor.IndexPosition.AfterIndex
                );
        savePdfFileToDisk("movefirstPDFPageToMiddle", pdfEditor);
    }

    /*
    It is possible to delete pages from a PDF file.
    Although at first, a PDF file looks as if it is set in stone but that's not
    the case. Not at least when you have a capable API at hand, like PS PDF.

    The process will be almost the same as duplicating PDF pages.

    High-level steps of this demo are given below:
    Open a PDF file from the assets included in the project(playground.pdf).
    Create an editor to modify the opened PDF in
    memory.
    Remove the page of the PDF using DocumentEditor.removePages method.
    Save the results in a new PDF file to the disk as a new PDF file.

    * */
    @Test
    public void removePDFPages() // a.k.a deletePDFPages
    {
        FileDataProvider providerPDFFile = new FileDataProvider(
                new File("Assets/playground.pdf"));
        PdfDocument playGround = PdfDocument.open(providerPDFFile);
        DocumentEditor pdfEditor = playGround.createDocumentEditor();

        Set<Integer> setOfPagesToRemove = new HashSet<Integer>(Arrays.asList(0, 2, 4));
        pdfEditor.removePages(setOfPagesToRemove);

        savePdfFileToDisk("fileWithRemovedPages", pdfEditor);
    }

    /*
    Extracting a new PDF from an existing PDF is a problem
    that can be solved using the removePages method.

    The process will be almost the same as duplicating PDF pages.
    High-level steps of this demo are given below:
    Open a PDF file from the assets included in the project(playground.pdf).
    Create an editor to modify the opened PDF in memory.
    Remove the page you don’t need in your new extracted PDF using DocumentEditor.removePages method.
    Save the results in a new PDF file to the disk as a new PDF file.

    * */
    @Test
    public void extractPDFSubtractive()
    {
        FileDataProvider providerPDFFile = new FileDataProvider(
                new File("Assets/playground.pdf"));
        PdfDocument playGround = PdfDocument.open(providerPDFFile);
        DocumentEditor pdfEditor = playGround.createDocumentEditor();

        Set<Integer> setOfPagesToRemoveFromExtractedPDF = new HashSet<Integer>(Arrays.asList(1, 3, 5));
        pdfEditor.removePages(setOfPagesToRemoveFromExtractedPDF);

        savePdfFileToDisk("extractPDF", pdfEditor);
    }

    /*
        It is possible to use a different method and specify which pages to keep
        instead of which pages to remove. We’re going to use the method
        DocumentEditor.keepPages for this purpose.
    */
    @Test
    public void extractPDFbyKeeping()
    {
        FileDataProvider providerPDFFile = new FileDataProvider(
                new File("Assets/playground.pdf"));
        PdfDocument playGround = PdfDocument.open(providerPDFFile);
        DocumentEditor pdfEditor = playGround.createDocumentEditor();

        Set<Integer> setOfPagesToKeepForPDFExtraction = new HashSet<Integer>(Arrays.asList(1, 3, 5));
        pdfEditor.keepPages(setOfPagesToKeepForPDFExtraction);

        savePdfFileToDisk("extractedPDFbyKeep", pdfEditor);
    }

    /*
        It is possible to add new page to an existing PDF file.
        We are going to use the DocumentEditor.addPage method for this purpose.
    */
    @Test
    public void addAPageToAPDF()
    {
        FileDataProvider providerPDFFile = new FileDataProvider(
                new File("Assets/playground.pdf"));
        PdfDocument playGround = PdfDocument.open(providerPDFFile);
        DocumentEditor pdfEditor = playGround.createDocumentEditor();

        Insets insets = new Insets(0, 0, 0, 0);

        pdfEditor.addPage(0, DocumentEditor.IndexPosition.BeforeIndex,
                200, 200, Rotation.Degrees90, Color.red, insets);

        savePdfFileToDisk("pageAddedToPDF", pdfEditor);
    }

/*
    How to split a PDF file
    Splitting a PDF file can be achieved using the technique shown in this blog post under the heading “Remove pages from a PDF file”.
    Essentially, you’ll create two DocumentEditor objects based on the same file. For the first object, you’ll specify which pages to keep and for the second you’ll specify the same pages to remove.
    In the end, you’ll save both files to the disc and you will have two PDF files “split” from a single PDF.

    High-level steps of this demo are given below:
    Open a PDF file from the assets included in the project(playground.pdf).
    Create an editor to modify the opened PDF in memory and keep a set of pages.
    Create another editor based on the same file and remove the same set of pages.
    Create an object of Set<Integer> and add page numbers that you want to be
    keep in the first DocumentEditor and removed from the second DocumentEditor.
    Page values start from 0 instead of 1.
    Make the method call to keep the set of pages on first DocumentEditor.
    Make the method call to remove the set of pages from the second
    DocumentEditor.
    Save a file based on the first DocumentEditor in a new PDF file
    to the disk as a new PDF file.
    Save a file based on the second DocumentEditor in a new PDF file
    to the disk as a new PDF file
* */
    @Test
    public void splitPDFFile()
    {
        FileDataProvider providerPDFFile = new FileDataProvider(
                new File("Assets/playground.pdf"));
        PdfDocument playGround = PdfDocument.open(providerPDFFile);
        DocumentEditor pdfEditorFirst = playGround.createDocumentEditor();
        DocumentEditor pdfEditorSecond = playGround.createDocumentEditor();
        Set<Integer> setOfPagesForSplit = new HashSet<Integer>(Arrays.asList(0, 1 , 2, 3));

        pdfEditorFirst.keepPages(setOfPagesForSplit);
        pdfEditorSecond.removePages(setOfPagesForSplit);

        savePdfFileToDisk("bananaSplitPDFFirst", pdfEditorFirst);
        savePdfFileToDisk("bananaSplitPDFSecond", pdfEditorSecond);
    }
/*
* Changing PDF page label
PDFs have a unique ability known as page labels. By default,
the page numbers are considered page labels. But, it is also possible
to change the  values of the page labels to something else.
Maybe the first few pages of a PDF file need to be labelled differently
from the rest of the e-book/PDF for example: A, B, C, D.
We could change the page labels accordingly.

High-level steps of this demo are given below:
Open a PDF file from the assets included in the project(playground.pdf).
Create an editor to modify the opened PDF in memory.
Create a set object to represent the pages for which we want to change
the label.
Call the method DocumentEditor.setPageLabel with the set of pages and
the label which we want to set.
Repeat steps 3-4 for all page numbers where we intend to modify.
*/
    @Test
    public void changePageLabels() {
        FileDataProvider providerPDFFile = new FileDataProvider(
                new File("Assets/playground.pdf"));
        PdfDocument playGround = PdfDocument.open(providerPDFFile);
        DocumentEditor pdfEditor = playGround.createDocumentEditor();

        Set<Integer> setOfPagesForChangingLabel = new HashSet<Integer>(Arrays.asList(0));
        pdfEditor.setPageLabel(setOfPagesForChangingLabel, "A");

        setOfPagesForChangingLabel = new HashSet<Integer>(Arrays.asList(1));
        pdfEditor.setPageLabel(setOfPagesForChangingLabel, "B");

        setOfPagesForChangingLabel = new HashSet<Integer>(Arrays.asList(2));
        pdfEditor.setPageLabel(setOfPagesForChangingLabel, "C");

        setOfPagesForChangingLabel = new HashSet<Integer>(Arrays.asList(3));
        pdfEditor.setPageLabel(setOfPagesForChangingLabel, "D");

        savePdfFileToDisk("pdfWithUpdatedLabels", pdfEditor);
    }

    @Test
    public void chainOfOperationsOnPDF()
    {
        FileDataProvider providerPDFFile = new FileDataProvider(
                new File("Assets/playground.pdf"));
        PdfDocument playGround = PdfDocument.open(providerPDFFile);
        DocumentEditor pdfEditor = playGround.createDocumentEditor();

        String operations = "";
        try {
            operations = new String(
                    Files.readAllBytes(
                            Paths.get("Assets/chain.json")));
        } catch (IOException e) {
            e.printStackTrace();
        }

        System.out.println(operations);

        JSONArray jsonOperations = new JSONArray(operations);

        pdfEditor.addOperations(jsonOperations);

        savePdfFileToDisk("chainOfOps", pdfEditor);

    }

}
